#include <FastLED.h>
#include <DS3232RTC.h>
#define LED_PIN     6
#define NUM_LEDS    145
#define BRIGHTNESS  55
DS3232RTC myRTC(false);
tmElements_t tm;
CRGB leds[NUM_LEDS];
CHSV colour( 0, 255, 180);

//various bits for nonblocking code
#define framerate 33 //how many milliseconds to wait between updating the display
unsigned long previousdisplaymillis = 0; //This stores the time when the LED display was last updated
#define backgroundframerate 50 // how many ms to wait between updating the background animation
unsigned long previousbackgroundmillis = 0; //When was the background last updated
#define timeupdate 1000 //how many ms to wait before checking & updating the time
unsigned long previoustimemillis = 0;
unsigned long currentmillis = 0;

//the LED display translaton array, first number is the row number, second the column number; values are the corresponding LED light for each of tehse positions
const byte LEDarray[23][9] =  {
  {145, 145, 145, 145, 0, 145, 145, 145, 145} ,
  {145, 145, 1, 2, 3, 4, 5, 145, 145} ,
  {145, 12, 11, 10, 9, 8, 7, 6, 145},
  {145, 13, 14, 15, 16, 17, 18, 19, 145} ,
  {145, 26, 25, 24, 23, 22, 21, 20, 145} ,
  {145, 27, 28, 29, 30, 31, 32, 33, 145} ,
  {145, 40, 39, 38, 37, 36, 35, 34, 145} ,
  {145, 41, 42, 43, 44, 45, 46, 47, 145} ,
  {145, 54, 53, 52, 51, 50, 49, 48, 145} ,
  {145, 55, 56, 57, 58, 59, 60, 61, 145} ,
  {145, 68, 67, 66, 65, 64, 63, 62, 145} ,
  {145, 69, 70, 71, 72, 73, 74, 75, 145} ,
  {145, 82, 81, 80, 79, 78, 77, 76, 145} ,
  {145, 83, 84, 85, 86, 87, 88, 89, 145} ,
  {145, 96, 95, 94, 93, 92, 91, 90, 145} ,
  {145, 97, 98, 99, 100, 101, 102, 103, 145} ,
  {145, 110, 109, 108, 107, 106, 105, 104, 145} ,
  {145, 111, 112, 113, 114, 115, 116, 117, 145} ,
  {145, 124, 123, 122, 121, 120, 119, 118, 145} ,
  {145, 125, 126, 127, 128, 129, 130, 131, 145} ,
  {145, 138, 137, 136, 135, 134, 133, 132, 145} ,
  {145, 145, 139, 140, 141, 142, 143, 145, 145} ,
  {145, 145, 145, 145, 144, 145, 145, 145, 145}
};

// The font used for the digit display
const int Digits[10][15] = //each digit (0-9) is a list of the relative sprite positions to light up (3x5 sprite)
{
  {6, 2, 12, 3, 13, 4, 14, 10}, //0
  {6, 2, 7, 8, 9, 5, 10, 15}, //1
  {1, 6, 12, 8, 4, 5, 10, 15}, //2
  {1, 6, 0, 12, 8, 0, 14, 5, 10}, //3
  {11, 2, 12, 3, 13, 4, 9, 14, 15}, //4
  {1, 6, 11, 2, 3, 8, 13, 14, 5, 10}, //5
  {6, 2, 3, 8, 4, 14, 10}, //6
  {1, 6, 11, 12, 8, 9, 10}, //7
  {6, 2, 12, 8, 4, 14, 10}, //8
  {6, 2, 12, 8, 13, 14, 10} //9
};

//Variables used to display the time
uint8_t hr10;
uint8_t hr1;
uint8_t min10;
uint8_t min1;

void setup()
{
  Serial.begin (9600); //debug
  delay(1000);
  pinMode(17, OUTPUT); //ground and v5 for clock module
  pinMode(16, OUTPUT);
  digitalWrite(17, HIGH);
  digitalWrite(16, LOW);
  FastLED.addLeds<WS2812, LED_PIN, GRB>(leds, NUM_LEDS);
  myRTC.begin();
}

void loop() {

  currentmillis = millis();
  if ((currentmillis - previoustimemillis) >= timeupdate) {
    showtime();
  }

  if ((currentmillis - previousdisplaymillis) >= framerate) {
    FastLED.show();
    //updatebackground();
    previousdisplaymillis = millis();
  }
}

void showtime() { //displays the time
  dimpreviousnumbersprite(3, hr10);
  dimpreviousnumbersprite(7, hr1);
  dimpreviousnumbersprite(13, min10);
  dimpreviousnumbersprite(17, min1);
  RTC.read(tm);
  hr10 = tm.Hour / 10;
  hr1 = tm.Hour % 10;
  min10 = tm.Minute / 10;
  min1 = tm.Minute % 10;
  displaynumbersprite(3, hr10);
  displaynumbersprite(7, hr1);
  displaynumbersprite(13, min10);
  displaynumbersprite(17, min1);
  previoustimemillis = millis();
}

void displaynumbersprite(uint8_t place, uint8_t number) { //Used to light up the individual LEDs in a digit
  //light up the LEDs for the current digit
  for (uint8_t i = 0; i < 15; i++) {
    uint8_t value = Digits[number][i];
    value--;
    uint8_t x = (value / 5) + place;
    uint8_t y = (value % 5) + 2;
    if (value > 200) { //pixels with a value of 0 end up at 200, which is outside the display
      x = 0;
      y = 0;
    }
    uint8_t r = (LEDarray [x][y]);
    //leds[r] = -leds[r];
    leds[r] = CRGB::HotPink;
  }

  //display colons
  if ( tm.Second % 2 == 0 ) {
    leds[71] = CRGB(0, 0, 255);
    leds[73] = CRGB(255, 0, 0);
  }
  else {
    leds[71] = CRGB(255, 0, 0);
    leds[73] = CRGB(0, 0, 255);
  }
}

void dimpreviousnumbersprite(uint8_t place, uint8_t number) { //Used to set any unused LEDs back to black
  //dim the LEDs for the previous digit
  for (uint8_t i = 0; i < 15; i++) {
    uint8_t value = Digits[number][i];
    value--;
    uint8_t x = (value / 5) + place;
    uint8_t y = (value % 5) + 2;
    if (value > 200) {
      x = 0;
      y = 0;
    }
    uint8_t r = (LEDarray [x][y]);
    leds[r] -= CRGB (255, 255, 255);
  }
}
